﻿' ---------------------------------------------------------
' Excel-Automatisierungsbeispiele
' Office 2007-Programmierung, Microsoft Press
' ---------------------------------------------------------
Imports Ex = Microsoft.Office.Interop.Excel
Imports Office = Microsoft.Office.Core
Imports System.Runtime.InteropServices

Public Class Form1
    Private ExApp As Ex.Application

    ' Startet Excel und legt eine neue Arbeitsmappe an
    Sub SetupExApp()
        Try
            ExApp = New Ex.Application
            ExApp.Visible = True
            ExApp.Workbooks.Add()
        Catch ex As SystemException
            MessageBox.Show(ex.Message, My.Application.Info.Title)
        End Try
    End Sub

    Private Sub bnArrayListInRangeKopieren_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles bnArrayListInRangeKopieren.Click
        SetupExApp()
        ' Kopieren einer ArrayList in ein Range
        Try
            Dim MeineFillialen As New ArrayList
            MeineFillialen.Add("Filiale Ost")
            MeineFillialen.Add("Filiale West")
            MeineFillialen.Add("Filiale Süd")
            MeineFillialen.Add("Filiale Nord")
            '  Typ muss "in etwa" passen
            Dim HilfsFeld As Array = Array.CreateInstance(GetType(String), 4)
            ' Es werden nur die Werte kopiert.
            MeineFillialen.CopyTo(HilfsFeld, 0)
            ExApp.Range("A1", "D1").Value2 = HilfsFeld
        Catch ex As SystemException
            MessageBox.Show(ex.Message, My.Application.Info.Title)
        End Try
    End Sub

    Private Sub bnBereicheKombinieren_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles bnBereicheKombinieren.Click
        SetupExApp()
        Dim R1 As Ex.Range = ExApp.Range("A1:A10")
        R1.Value = 100
        Dim R2 As Ex.Range = ExApp.Range("A1:J1")
        R2.Value = 200
        Dim R3 As Ex.Range = ExApp.Union(R1, R2)
        MessageBox.Show("Anzahl Zellen: " & R3.Cells.Count, My.Application.Info.Title)
    End Sub

    Private Sub bnBereichDurchsuchen_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles bnBereichDurchsuchen.Click
        SetupExApp()
        Dim SuchBereich As Ex.Range = ExApp.Range("A1:J1")
        ' Bereich mit Zufallszahlen füllen
        For n As Short = 1 To 10
            SuchBereich.Cells(ColumnIndex:=n).Value2 = New Random().Next(100, 105)
        Next
        Dim AnzahlGefunden As Short = 0
        ' Gesucht wird nach der Zahl 104
        Dim R As Ex.Range = SuchBereich.Find(What:=104, LookIn:=Ex.XlFindLookIn.xlValues)
        If R IsNot Nothing Then
            Dim StartAdresse As String = R.Address
            Do
                AnzahlGefunden += 1
                R = SuchBereich.FindNext(R)
            Loop Until R Is Nothing Or R.Address = StartAdresse
        End If
        MessageBox.Show("Anzahl gefunden:  " & AnzahlGefunden, My.Application.Info.Title)
    End Sub

    Private Sub bnExcelFormel_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles bnExcelFormel.Click
        SetupExApp()
        Try
            Dim Sh As Ex.Worksheet = ExApp.Worksheets("Tabelle1")
            Sh.Names.Add(Name:="Preis", RefersTo:="=$A$1")
            Sh.Names.Add(Name:="Menge", RefersTo:="=$B$1")
            Sh.Names.Add(Name:="Rabatt", RefersTo:="=$C$1")
            Sh.Range("Preis").Value = 9.9
            Sh.Range("Menge").Value = 10
            Sh.Range("Rabatt").Value = 20
            With Sh.Range("A2")
                .Value = "=Preis*Menge*(1-Rabatt/100)"
                .NumberFormat = "#,##0.00 $"
            End With
        Catch Ex As SystemException
            MessageBox.Show(Ex.Message, My.Application.Info.Title)
        End Try
    End Sub

    Private Sub bnZellenFormat_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles bnZellenFormat.Click
        SetupExApp()
        Dim Sh As Ex.Worksheet = ExApp.ActiveSheet
        For i As Short = 1 To 10
            Sh.Cells.Item(1, i).Value = i
            With CType(Sh.Cells.Item(1, i), Ex.Range)
                .Interior.ColorIndex = i + 2
                .Font.Bold = True
            End With
        Next
    End Sub

    Private Sub bnBalkendiagramm1_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles bnBalkendiagram1.Click
        Dim C As Ex.Chart
        Dim R As Ex.Range
        Dim Sh As Ex.Worksheet
        SetupExApp()
        Try
            Sh = ExApp.ActiveSheet
            For i As Short = 1 To 12
                Sh.Cells(1, i).Value = New Random(Now.Millisecond).Next(1, 50) + 50
            Next
            C = ExApp.Charts.Add
            C.ChartType = Ex.XlChartType.xl3DColumn
            R = Sh.Range("A1:L12")
            C.ChartWizard(Source:=R, Title:="Umsatz 2008")
            With CType(C.SeriesCollection(1), Ex.Series).Fill
                .TwoColorGradient(Style:=Microsoft.Office.Core.MsoGradientStyle.msoGradientHorizontal, Variant:=1)
                .ForeColor.SchemeColor = 3
                .BackColor.SchemeColor = 17
            End With
        Catch Ex As COMException
            MessageBox.Show("COMException:" & Ex.Message, My.Application.Info.Title)
        Catch Ex As SystemException
            MessageBox.Show(Ex.Message, My.Application.Info.Title)
        End Try
    End Sub

    Private Sub bnBalkendiagramm2_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles bnKuchendiagramm2.Click
        SetupExApp()
        Dim Sh As Ex.Worksheet
        Dim NewChartObj As Ex.ChartObject
        Dim C As Ex.Chart
        Dim R As Ex.Range
        Dim SCol As Ex.SeriesCollection
        Try
            Sh = ExApp.ActiveSheet
            ' Zufallszahlen einfüllen
            For i As Short = 1 To 12
                Sh.Cells(1, i).Value = New Random(Now.Millisecond).Next(1, 50) + 50
            Next
            ' Chart zu den eingebetteten Charts hinzufügen
            NewChartObj = Sh.ChartObjects.Add(10, 30, 500, 300)
            C = NewChartObj.Chart
            R = Sh.Range("A1:L1")
            With C
                .SetSourceData(R)
                .ChartType = Ex.XlChartType.xl3DColumn
            End With
            SCol = C.SeriesCollection
            Dim DatenReihe1 As Ex.Series = C.SeriesCollection.NewSeries
            DatenReihe1.Values = _
               New Short() {100, 120, 80, 160, 240, 80, 90, 110, 88, 124, 64, 100}
            Dim DatenReihe2 As Ex.Series = C.SeriesCollection.NewSeries
            DatenReihe2.Values = _
               New Short() {200, 220, 180, 160, 240, 180, 190, 210, 188, 224, 164, 200}
            SCol.Item(2).Interior.ColorIndex = 4
            SCol.Item(3).Interior.ColorIndex = 8
        Catch Ex As COMException
            MessageBox.Show("COMException:" & Ex.Message, My.Application.Info.Title)
        Catch Ex As Exception
            MessageBox.Show(Ex.Message, My.Application.Info.Title)
        End Try
    End Sub

    Private Sub bnLinienDiagramm_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles bnLinienDiagramm.Click
        ' Excel starten und Application anlegen
        SetupExApp()
        Dim Ch As Ex.Chart
        Dim Sh As Ex.Worksheet = ExApp.ActiveSheet
        Dim Wb As Ex.Workbook = ExApp.ActiveWorkbook
        ' Zufallszahlen einfüllen
        For i As Short = 1 To 12
            Sh.Cells(1, i).Value = New Random(Now.Millisecond).Next(1, 50) + 50
        Next
        Try
            Dim Daten As Ex.Range = Sh.Range(Sh.Cells(1, 1), Sh.Cells(1, 12))
            Ch = Wb.Charts.Add
            With Ch
                .ChartType = Ex.XlChartType.xlLineMarkers
                .SetSourceData(Source:=Daten)
                .Location(Ex.XlChartLocation.xlLocationAsObject, Sh.Name)
            End With
            Dim Cho As Ex.ChartObject
            Cho = Sh.ChartObjects(1)
            With Cho.Chart
                .HasTitle = True
                .ChartTitle.Text = "Ein neuer Titel"
            End With
            Cho.Interior.ColorIndex = 4
            Cho.Interior.ColorIndex = 4
            Cho.Interior.ColorIndex = 4
            ' Hat scheinbar keine Wirkung
            Cho.Chart.SeriesCollection(1).Interior.ColorIndex = 2
        Catch Ex As COMException
            MessageBox.Show("COMException: " & Ex.Message, My.Application.Info.Title)
        Catch Ex As SystemException
            MessageBox.Show("SystemException: " & Ex.Message, My.Application.Info.Title)
        End Try
    End Sub

    Private Sub bnTextboxShape_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles bnTextboxShape1.Click
        ' Excel starten und Application anlegen
        SetupExApp()
        Dim Sh As Ex.Worksheet = ExApp.ActiveSheet
        Dim ShText As Ex.Shape
        Dim Start As Ex.Range = ExApp.Range("C2")
        Dim Breite As Integer = 200
        Dim Hoehe As Integer = 18
        Dim Inhalt As String = "Verkaufszahlen 2008"
        Try
            ShText = Sh.Shapes.AddTextbox(Office.MsoTextOrientation.msoTextOrientationHorizontal, _
                Start.Left, Start.Top, Breite, Hoehe)
            With ShText
                .TextFrame.Characters.Text = Inhalt
                .TextFrame.HorizontalAlignment = Ex.XlHAlign.xlHAlignCenter
                .Fill.Solid()
                .Fill.ForeColor.RGB = RGB(232, 232, 240)
                .Line.Visible = Office.MsoTriState.msoCTrue
            End With
        Catch Ex As COMException
            MessageBox.Show("COMException: " & Ex.Message, My.Application.Info.Title)
        Catch Ex As SystemException
            MessageBox.Show("SystemException: " & Ex.Message, My.Application.Info.Title)
        End Try
    End Sub

    Private Sub bnTextbox2_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles bnTextboxShape2.Click
        ' Excel starten und Application anlegen
        SetupExApp()
        Try
            Dim ShText As Ex.Shape
            Dim Sh As Ex.Worksheet = ExApp.ActiveSheet
            Dim Inhalt As String = "ATTENTION: Datenglatteis"
            Dim Suchwort As String = "ATTENTION:"
            ShText = Sh.Shapes.AddTextbox(Office.MsoTextOrientation.msoTextOrientationHorizontal, 10, 10, 200, 50)
            ShText.TextFrame.Characters.Text = Inhalt
            ShText.Fill.ForeColor.RGB = RGB(234, 234, 234)
            With ShText
                With .TextFrame.Characters(.TextFrame.Characters.Text.IndexOf(Suchwort, 0), Suchwort.Length).Font
                    .ColorIndex = 3
                    .Bold = True
                    ' Lässt sich nicht setzen
                    .Underline = True
                End With
            End With
        Catch Ex As COMException
            MessageBox.Show("COMException: " & Ex.Message, My.Application.Info.Title)
        Catch Ex As SystemException
            MessageBox.Show("SystemException: " & Ex.Message, My.Application.Info.Title)
        End Try
    End Sub

    Private Sub bnBildEinfuegen_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles bnBildEinfuegen.Click
        ' Excel starten und Application anlegen
        SetupExApp()
        Try
            Dim LogoPfad As String = New System.IO.DirectoryInfo(Environment.GetEnvironmentVariable("systemroot")).GetFiles("*.bmp")(0).FullName
            Dim Sh As Ex.Worksheet = ExApp.ActiveSheet
            LogoSetzen(Sh, 10, 10, 50, 50, LogoPfad)
        Catch Ex As COMException
            MessageBox.Show("COMException: " & Ex.Message, My.Application.Info.Title)
        Catch Ex As SystemException
            MessageBox.Show("SystemException: " & Ex.Message, My.Application.Info.Title)
        End Try
    End Sub

    Sub LogoSetzen(ByVal Sh As Ex.Worksheet, ByVal Top As Integer, ByVal Left As Integer, _
ByVal Breite As Integer, ByVal Hoehe As Integer, ByVal LogoPfad As String)
        Try
            Dim Pic As Ex.Picture
            Pic = Sh.Pictures.Insert(LogoPfad)
            With Pic
                .Top = Top
                .Left = Left
                .Width = Breite
                .Height = Hoehe
            End With
        Catch Ex As COMException
            MessageBox.Show("COMException: " & Ex.Message, My.Application.Info.Title)
        Catch Ex As SystemException
            MessageBox.Show("SystemException: " & Ex.Message, My.Application.Info.Title)
        End Try
    End Sub

    Private Sub bnLiniendiagrammMarkierungen_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles bnLiniendiagrammMarkierungen.Click
        ' Excel starten und Application anlegen
        SetupExApp()
        Try
            Dim Sh As Ex.Worksheet = ExApp.ActiveSheet
            Dim Ch As Ex.Chart = CType(CType(Sh.ChartObjects, Ex.ChartObjects).Add(10, 40, 400, 200),  _
                    Ex.ChartObject).Chart
            Dim Zahlenfeld(10) As Byte
            Dim R As New Random
            R.NextBytes(Zahlenfeld)
            For i As Integer = 1 To 10
                Sh.Cells(1, i).Value = Zahlenfeld(i)
            Next
            Dim Daten As Ex.Range = Sh.Range(Sh.Cells(1, 1), Sh.Cells(1, 10))
            With Ch
                .ChartType = Ex.XlChartType.xlLineMarkers
                .SetSourceData(Daten)
                With .SeriesCollection(1)
                    .Name = "Messwerte"
                    .MarkerStyle = 8
                    .MarkerSize = 6
                    .MarkerforegroundColorIndex = 4
                    .MarkerbackgroundColorIndex = 4
                    .Border.Weight = Ex.XlBorderWeight.xlMedium
                    .Border.ColorIndex = 5
                End With
                ' Zum Schluss noch den Titel formatieren
                With .ChartTitle
                    .Font.Size = 12
                    .Font.Bold = True
                End With
                ' Und die Legende etwas kleiner
                With .Legend
                    .Font.Size = 9
                End With
            End With
        Catch Ex As COMException
            MessageBox.Show("COMException: " & Ex.Message, My.Application.Info.Title)
        Catch Ex As SystemException
            MessageBox.Show("SystemException: " & Ex.Message, My.Application.Info.Title)
        End Try
    End Sub

    Private Sub bnLiniendiagrammMitBitmap_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles bnLiniendiagrammMitBitmap.Click
        ' Excel starten und Application anlegen
        SetupExApp()
        Try
            Dim Sh As Ex.Worksheet = ExApp.ActiveSheet
            Dim Ch As Ex.Chart = CType(CType(Sh.ChartObjects, Ex.ChartObjects).Add(10, 40, 400, 200),  _
                    Ex.ChartObject).Chart
            Dim Zahlenfeld(10) As Byte
            Dim R As New Random
            R.NextBytes(Zahlenfeld)
            For i As Integer = 1 To 10
                Sh.Cells(1, i).Value = Zahlenfeld(i)
            Next
            Dim Daten As Ex.Range = Sh.Range(Sh.Cells(1, 1), Sh.Cells(1, 10))
            Dim Pi As New Bitmap(Environment.GetFolderPath(Environment.SpecialFolder.MyPictures) & "\Smiley.png")
            Clipboard.SetDataObject(Pi)
            With Ch
                .ChartType = Ex.XlChartType.xlLineMarkers
                .SetSourceData(Daten)
                With CType(.SeriesCollection(1), Ex.Series)
                    .Name = "Messwerte"
                    .MarkerStyle = 8
                    .MarkerSize = 6
                    .MarkerForegroundColorIndex = 4
                    .MarkerBackgroundColorIndex = 4
                    .Border.Weight = Ex.XlBorderWeight.xlMedium
                    .Border.ColorIndex = 5
                    For Each M As Ex.Point In .Points
                        M.Paste()
                    Next
                End With
                ' Zum Schluss noch den Titel formatieren
                With .ChartTitle
                    .Font.Size = 12
                    .Font.Bold = True
                End With
                ' Und die Legende etwas kleiner
                With .Legend
                    .Font.Size = 9
                End With
            End With
        Catch Ex As COMException
            MessageBox.Show("COMException: " & Ex.Message, My.Application.Info.Title)
        Catch Ex As SystemException
            MessageBox.Show("SystemException: " & Ex.Message, My.Application.Info.Title)
        End Try

    End Sub

    Private Sub bnCSVImport_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles bnCSVImport.Click
        ' Excel starten und Application anlegen
        SetupExApp()
        Try
            Dim Sh As Ex.Worksheet
            Using Ofd As New OpenFileDialog()
                With Ofd
                    .Filter = "Csv-Dateien (*.Csv)|*.csv|Alle Dateien|*.*"
                    .InitialDirectory = My.Application.Info.DirectoryPath
                    Dim ExWin As New NativeWindow()
                    ExWin.AssignHandle(Process.GetProcessesByName("Excel")(0).MainWindowHandle)
                    If .ShowDialog(ExWin) = Windows.Forms.DialogResult.OK Then
                        Dim xlText As Ex.XlColumnDataType = Ex.XlColumnDataType.xlTextFormat
                        Dim TestArray(,) As Object = New Object(,) {{1, xlText}, {1, xlText}, {1, xlText}, _
                            {1, xlText}, {1, xlText}}
                        ExApp.Workbooks.OpenText(Filename:=.FileName, DataType:=Ex.XlTextParsingType.xlDelimited, _
                            Comma:=True, FieldInfo:=TestArray)
                        Sh = ExApp.ActiveSheet
                        Dim Li As Ex.ListObject = Sh.ListObjects.Add(Ex.XlListObjectSourceType.xlSrcRange, _
                            Sh.Range("A1:E5"), , Ex.XlYesNoGuess.xlYes)
                        Li.ShowTotals = True
                        Li.ListColumns(1).Total.Value2 = "Summe"
                        For i As Short = 2 To 4
                            Li.ListColumns(i).TotalsCalculation = Ex.XlTotalsCalculation.xlTotalsCalculationSum
                        Next
                    End If
                End With
            End Using
        Catch Ex As COMException
            MessageBox.Show("COMException: " & Ex.Message, My.Application.Info.Title)
        Catch Ex As SystemException
            MessageBox.Show("SystemException: " & Ex.Message, My.Application.Info.Title)
        End Try
    End Sub

    ' -------------------------------------------------
    ' Beispiel XML-Import in ein Excel-Tabellenblatt
    ' -------------------------------------------------
    Private Sub bnXMLImport_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles bnXMLImport.Click
        ' Excel starten und Application anlegen
        SetupExApp()
        Dim Wb As Ex.Workbook
        Dim Liste As Ex.ListObject
        Try
            Using Ofd As New OpenFileDialog()
                With Ofd
                    .Filter = "Xml-Dateien (*.xml)|*.xml|Alle Dateien|*.*"
                    .InitialDirectory = My.Application.Info.DirectoryPath
                    Dim ExWin As New NativeWindow()
                    ExWin.AssignHandle(New IntPtr(ExApp.Hwnd))
                    If .ShowDialog(ExWin) = Windows.Forms.DialogResult.OK Then
                        Wb = ExApp.ActiveWorkbook
                        ExApp.DisplayAlerts = False
                        Wb.XmlImport(.FileName, Nothing, True, Wb.ActiveSheet.Range("A1"))
                        ' Die eingefügte Liste heißt bei Excel 2003 Liste1, bei Excel 2007 Tabelle1
                        Liste = ExApp.ActiveSheet.ListObjects(1)
                        Liste.ShowTotals = True
                        Liste.TotalsRowRange.Cells(ColumnIndex:=1).Value2 = "Durchschnitt"
                        Liste.ListColumns("AnzahlLogins").TotalsCalculation = _
                            Ex.XlTotalsCalculation.xlTotalsCalculationAverage
                    End If
                    ExWin.ReleaseHandle()
                End With
            End Using
        Catch Ex As COMException
            MessageBox.Show("COMException: " & Ex.Message, My.Application.Info.Title)
        Catch Ex As SystemException
            MessageBox.Show("SystemException: " & Ex.Message, My.Application.Info.Title)
        End Try
    End Sub

    Private Sub bnBereichInListe_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles bnBereichInListe.Click
        ' Excel starten und Application anlegen
        SetupExApp()
        Dim Wb As Ex.Workbook = ExApp.ActiveWorkbook
        Dim Liste As Ex.ListObject
        Try
            With ExApp.ActiveSheet
                .Range("A1").Value = "Logins"
                .Range("A2").Value = 20
                .Range("A3").Value = 30
                .Range("A4").Value = 40
                .Range("A5").Value = 50
            End With
            Liste = ExApp.ActiveSheet.ListObjects.Add(1, Wb.ActiveSheet.Range("A1:A5"))
            With Liste
                .ShowTotals = True
                .ListColumns("Logins").TotalsCalculation = Ex.XlTotalsCalculation.xlTotalsCalculationAverage
            End With
        Catch Ex As COMException
            MessageBox.Show("COMException: " & Ex.Message, My.Application.Info.Title)
        Catch Ex As SystemException
            MessageBox.Show("SystemException: " & Ex.Message, My.Application.Info.Title)
        End Try
    End Sub

    Private Sub bnDiagrammPlatzieren_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles bnDiagrammPlatzieren.Click
        ' Excel starten und Application anlegen
        SetupExApp()
        Dim Ch As Ex.Chart
        Dim Wb As Ex.Workbook = ExApp.ActiveWorkbook
        Dim Sh As Ex.Worksheet = ExApp.ActiveSheet
        Dim Shp As Ex.ShapeRange
        ' Zufallszahlen einfüllen
        For i As Short = 1 To 12
            Sh.Cells(1, i).Value = New Random(Now.Millisecond).Next(1, 50) + 50
        Next
        Try
            Dim Daten As Ex.Range = Sh.Range(Sh.Cells(1, 1), Sh.Cells(1, 12))
            Ch = Wb.Charts.Add
            With Ch
                .ChartType = Ex.XlChartType.xlLineMarkers
                .SetSourceData(Source:=Daten)
                .Location(Ex.XlChartLocation.xlLocationAsObject, Sh.Name)
            End With
            Dim Cho As Ex.ChartObject
            Cho = Sh.ChartObjects(1)
            With Cho.Chart
                .HasTitle = True
                .ChartTitle.Text = "Dies ist der Titel"
            End With
            Shp = Cho.ShapeRange
            DiagrammPlatzieren(Shp, Sh.Range("A3"), 320, 200)
        Catch Ex As COMException
            MessageBox.Show("COMException: " & Ex.Message, My.Application.Info.Title)
        Catch Ex As SystemException
            MessageBox.Show("SystemException: " & Ex.Message, My.Application.Info.Title)
        End Try
    End Sub

    Sub DiagrammPlatzieren(ByVal Shp As Ex.ShapeRange, ByVal R As Ex.Range, ByVal Breite As Single, ByVal Hoehe As Single)
        With Shp
            .Left = R.Left
            .Top = R.Top
            .Width = Breite
            .Height = Hoehe
        End With
    End Sub

End Class
