﻿' ---------------------------------------------------------
' Zugriff auf SharePoint-Listen und ihre Inhalte über Webservices
' Das Office 2007-Entwicklerhandbuch
' ---------------------------------------------------------
Imports System.Xml
Imports System.Windows.Forms
Imports System.Net

Public Class SPExplorer
    ' Die Credentials für die Anmeldung
    Dim SPCred As System.Net.NetworkCredential
    ' Die SharePoint-Listen des Webservice
    Dim SPLists As SPListen.Lists
    ' Die Ansichten einer Liste
    Dim SPViews As SPViews.Views

    Private Sub bnVerbinden_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles bnVerbinden.Click
        Try
            SPCred.UserName = tbBenutzername.Text
            SPCred.Password = tbKennwort.Text

            SPLists.Url = tbURL.Text + "/_vti_bin/lists.asmx"
            SPViews.Url = tbURL.Text + "/_vti_bin/views.asmx"

            SPLists.Credentials = SPCred
            SPLists.Timeout = 10000
            SPViews.Credentials = SPCred
            SPViews.Timeout = 1000

            ' Die Listen durchlaufen
            Dim XnListen As XmlNode = SPLists.GetListCollection()
            liListen.Items.Clear()
            For Each XnListe As XmlNode In XnListen.ChildNodes
                ' Title-Attribut ansprechen
                liListen.Items.Add(XnListe.Attributes("Title").InnerText())
            Next
        Catch Ex As XmlException
            MessageBox.Show("Xml-Error:" & Ex.Message, My.Application.Info.Title)
        Catch ex As SystemException
            MessageBox.Show("System-Error:" & ex.Message, My.Application.Info.Title)
        End Try
    End Sub

    Private Sub SPExplorer_Load(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Load
        SPCred = New System.Net.NetworkCredential
        SPLists = New SPListen.Lists
        SPViews = New SPViews.Views
        tbBenutzername.Text = "Administrator"
        tbKennwort.PasswordChar = "*"
        tbKennwort.Text = ""
        tbURL.Text = "http://pmserver/websites/pemo2"
    End Sub

    Private Sub liListen_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles liListen.SelectedIndexChanged
        Try
            Dim AlleViewKnoten As XmlNode = SPViews.GetViewCollection(liListen.SelectedItem)
            liViews.Items.Clear()
            For Each ViewKnoten As XmlNode In AlleViewKnoten.ChildNodes
                ' Name der View über DisplayName-Attribut ansprechen
                liViews.Items.Add(ViewKnoten.Attributes("DisplayName").InnerText)
            Next
        Catch Ex As WebException
            MessageBox.Show("Web-Error:" & Ex.Message, My.Application.Info.Title)
        Catch Ex As SystemException
            MessageBox.Show("System-Error:" & Ex.Message, My.Application.Info.Title)
        End Try

    End Sub

    Private Sub bnUebernehmen_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles bnUebernehmen.Click
        Dim SPListItems As XmlNode = GetListItems()
        Dim TitelListe As New List(Of String)
        Try
            ' Alle Knoten durchgehen
            For Each SPListItem As XmlNode In SPListItems
                ' Es kommt auf die z:row-Knoten an
                For i As Integer = 0 To SPListItem.ChildNodes.Count - 1
                    If SPListItem.ChildNodes.Item(i).Name = "z:row" Then
                        Dim Titel As String = SPListItem.ChildNodes.Item(i).Attributes("ows_LinkTitleNoMenu").Value
                        TitelListe.Add(Titel)
                    End If
                Next
            Next
        Catch ex As SystemException
            ' Hier passiert nichts
        End Try
        Dim R As Excel.Range = Globals.ThisAddIn.Application.ActiveCell
        Dim ZeilenOffset As Integer = 2
        R.Value = liListen.Text
        For Each Titel As String In TitelListe
            R.Cells(RowIndex:=ZeilenOffset).Value = Titel
            ZeilenOffset += 1
        Next
    End Sub

    Function GetWebID(ByVal SPPfad As String) As Guid
        Dim SPSiteDataWs As New SPSiteData.SiteData()
        Dim WebMetaData As SPSiteData._sWebMetadata
        Dim WebWithTime() As SPSiteData._sWebWithTime
        Dim ListWithTime() As SPSiteData._sListWithTime
        Dim URL() As SPSiteData._sFPUrl
        Dim Roles As String
        Dim RolesUsers() As String
        Dim RoleGroups() As String
        Dim Ret As Integer
        Try
            SPSiteDataWs.Credentials = SPCred
            SPSiteDataWs.Url = tbURL.Text & "/_vti_bin/sitedata.asmx"
            Ret = SPSiteDataWs.GetWeb(WebMetaData, WebWithTime, ListWithTime, URL, Roles, RolesUsers, RoleGroups)
            Dim g As New Guid(WebMetaData.WebID)
            Return g
        Catch ex As SystemException
            Throw New SystemException("Fehler in GetWebID", ex)
        End Try
    End Function

    Function GetListItems() As XmlNode
        Dim ListWs As New SPListen.Lists
        Try
            ListWs.Url = tbURL.Text & "/_vti_bin/lists.asmx"
            ListWs.Credentials = SPCred
            Dim XDoc As New XmlDocument
            XDoc.LoadXml("<Document><Query/><ViewFields/><QueryOptions/></Document>")
            Dim ListQuery As XmlNode = XDoc.SelectSingleNode("//Query")
            Dim ListViewFields As XmlNode = XDoc.SelectSingleNode("//ViewFields")
            Dim ListQueryOptions As XmlNode = XDoc.SelectSingleNode("//QueryOptions")
            Dim g As Guid = GetWebID(tbURL.Text)
            Dim SpListItems As XmlNode = ListWs.GetListItems(liListen.Text, String.Empty, ListQuery, ListViewFields, String.Empty, ListQueryOptions, g.ToString())
            Return SpListItems
        Catch ex As SystemException
            Throw New SystemException("Fehler in GetListItems", ex)
        End Try
    End Function

End Class
