﻿' EIn paar Beispiele für den Einsatz der Policy-Klassen
Imports System.Security
Imports System.Security.Policy
Imports System.Security.Permissions
Imports System.Reflection

Module Module1

    Sub Main()
        While True
            Dim Antwort As Byte = ShowMenu()
            Select Case Antwort
                Case 49
                    UrlPolicyAnlegen()
                Case 50
                    StrongNamePolicyAnlegen()
                Case 51
                    RemoveUrlPolicy()
                Case 52
                    ListCodegroupStart()
                Case Else
                    Exit While
            End Select
            Console.ReadLine()
        End While
    End Sub

    Function ShowMenu() As Byte
        Console.WriteLine(New String("*", 48))
        Console.WriteLine("1. Codegruppe mit URL-Bedingung anlegen")
        Console.WriteLine("2. Codegruppe mit Strong Name-Bedingung anlegen")
        Console.WriteLine("3. Codegruppe entfernen")
        Console.WriteLine("4. Codegruppen auflisten")
        Console.WriteLine("5. Ende")
        Console.WriteLine(New String("*", 48))
        Dim Key As ConsoleKeyInfo = Console.ReadKey(True)
        Return Key.Key
    End Function

    Sub RemoveUrlPolicy()
        Dim GruppenName As String = "Kapitel 14-Policy"
        Dim PolLev As PolicyLevel = GetPolLevelUser()
        Dim All_Code_Group As UnionCodeGroup = DirectCast(PolLev.RootCodeGroup, UnionCodeGroup)
            Dim AlleCodeGroups As IList = All_Code_Group.Children
            For Each Cgr As CodeGroup In AlleCodeGroups
                If Cgr.Name = GruppenName Then
                Try
                    All_Code_Group.RemoveChild(Cgr)
                    SecurityManager.SavePolicy()
                    Console.WriteLine("Codegruppe " & GruppenName & " wurde entfernt.")
                    Exit Sub
                Catch ex As PolicyException
                    Console.ForegroundColor = ConsoleColor.Yellow
                    Console.WriteLine("Policy-Error: Codegruppe " & GruppenName & " konnte nicht entfernt werden.", ex.Message)
                    Console.ForegroundColor = ConsoleColor.White
                Catch ex As SystemException
                    Console.ForegroundColor = ConsoleColor.Yellow
                    Console.WriteLine("System-Error: Codegruppe " & GruppenName & " konnte nicht entfernt werden.", ex.Message)
                    Console.ForegroundColor = ConsoleColor.White
                End Try
            End If
        Next
        Console.WriteLine("Codegruppe " & GruppenName & " wurde nicht gefunden!")
    End Sub

    Sub UrlPolicyAnlegen()
        Dim AllCodeCodeGroup As CodeGroup
        Dim VSTOCodeGroup As UnionCodeGroup
        Dim VSTOPermSet As PermissionSet
        Dim VSTOAssUrl As String = "\\PMServer\VSTO\*"
        Try
            Dim PolLev As PolicyLevel = GetPolLevelUser()
            AllCodeCodeGroup = DirectCast(PolLev.RootCodeGroup, UnionCodeGroup)
            VSTOPermSet = PolLev.GetNamedPermissionSet("FullTrust")
            Dim UrlMemCond As New UrlMembershipCondition(VSTOAssUrl)
            Dim PolStm As New PolicyStatement(VSTOPermSet)
            VSTOCodeGroup = New UnionCodeGroup(UrlMemCond, PolStm)
            VSTOCodeGroup.Description = "Codegruppe aus Kapitel 14"
            VSTOCodeGroup.Name = "Kapitel 14-Policy"
            AllCodeCodeGroup.AddChild(VSTOCodeGroup)
            ' TODO: Besser wäre es, an die VSTO_Projects-Gruppe zu hängen
            ' Alle Ändungen speichern
            SecurityManager.SavePolicy()
            Console.WriteLine("Codegruppe wurde angelegt.")
        Catch Ex As PolicyException
            Console.ForegroundColor = ConsoleColor.Yellow
            Console.WriteLine("Policy-Error: " & Ex.Message)
            Console.ForegroundColor = ConsoleColor.White
        Catch ex As SystemException
            Console.ForegroundColor = ConsoleColor.Yellow
            Console.WriteLine("System-Error: " & ex.Message)
            Console.ForegroundColor = ConsoleColor.White
        End Try

    End Sub

    Sub StrongNamePolicyAnlegen()
        ' Nimmt eigenes Assembly an
        Dim Ass As Assembly = Assembly.GetExecutingAssembly
        Dim snName As String = Ass.GetName.Name
        Dim snVersion As System.Version = Ass.GetName.Version
        Dim snPK() As Byte
        Try
            snPK = Ass.GetName.GetPublicKey()
        Catch ex As SystemException
            Console.WriteLine("Fehler beim Lesen des Public Key.")
            Exit Sub
        End Try
        Try
            Dim snPKB As New StrongNamePublicKeyBlob(snPK)
            Dim StrongCond As StrongNameMembershipCondition = _
                New StrongNameMembershipCondition(snPKB, snName, snVersion)
            Dim VSTOPermSet As New NamedPermissionSet("FullTrust")
            Dim VSTOCodeGroup As UnionCodeGroup = New UnionCodeGroup(StrongCond, New PolicyStatement(VSTOPermSet))
            VSTOCodeGroup.Description = "StrongName-Codegruppe aus Kapitel 14"
            VSTOCodeGroup.Name = "Kapitel 14-Policy (Strong)"
            Dim PolLev As PolicyLevel = GetPolLevelUser()
            Dim AllCodeCodeGroup As CodeGroup = DirectCast(PolLev.RootCodeGroup, UnionCodeGroup)
            AllCodeCodeGroup.AddChild(VSTOCodeGroup)
            ' TODO: Besser wäre es, an die VSTO_Projects-Gruppe zu hängen
            ' Alle Ändungen speichern
            SecurityManager.SavePolicy()
            Console.WriteLine("StrongName-Codegruppe wurde angelegt.")
        Catch Ex As SecurityException
            Console.ForegroundColor = ConsoleColor.Yellow
            Console.WriteLine("Security-Error: " & Ex.Message)
            Console.ForegroundColor = ConsoleColor.White
        Catch Ex As PolicyException
            Console.ForegroundColor = ConsoleColor.Yellow
            Console.WriteLine("Policy-Error: " & Ex.Message)
            Console.ForegroundColor = ConsoleColor.White
        Catch ex As SystemException
            Console.ForegroundColor = ConsoleColor.Yellow
            Console.WriteLine("System-Error: " & ex.Message)
            Console.ForegroundColor = ConsoleColor.White
        End Try
    End Sub

    Sub ListCodegroupStart()
        Dim AnzahlCodeGruppen As Integer
        Try
            Dim PolLevel As PolicyLevel = GetPolLevelUser()
            For Each CG As CodeGroup In PolLevel.RootCodeGroup.Children
                ListCodeGroups(CG, AnzahlCodeGruppen)
            Next
            Console.ForegroundColor = ConsoleColor.Magenta
            Console.WriteLine("Anzahl Codegruppen: {0}", AnzahlCodeGruppen)
            Console.ForegroundColor = ConsoleColor.White
        Catch ex As SystemException
            ' Es trat ein Fehler auf
            Console.WriteLine("Error: " & ex.Message)
        End Try
    End Sub

    Sub ListCodeGroups(ByVal GC As CodeGroup, ByRef AnzahlCodeGruppen As Integer)
        Static Level As Integer
        Level += 1
        Console.WriteLine(New String("*", Level * 3) & "Codegruppe: " & GC.Name)
        AnzahlCodeGruppen += 1
        For Each GCSub As CodeGroup In GC.Children
            ListCodeGroups(GCSub, AnzahlCodeGruppen)
        Next
        Level -= 1
    End Sub

    ' Liefert das PolicyLevel-Objekt für die User-Ebene
    Private Function GetPolLevelUser() As PolicyLevel
        Try
            Dim PL As PolicyLevel
            Dim AllePolLevel As IEnumerator = SecurityManager.PolicyHierarchy()
            While AllePolLevel.MoveNext()
                Dim PolLevel As PolicyLevel = CType(AllePolLevel.Current, PolicyLevel)
                If PolLevel.Label = "User" Then
                    PL = PolLevel
                    Exit While
                End If
            End While
            If PL Is Nothing Then
                Throw New ApplicationException("User-Gruppe nicht gefunden - Probleme mit CAS-Konfiguration auf diesem Computer")
            End If
            Return PL
        Catch ex As SystemException
            ' Es trat ein Fehler auf
            Console.WriteLine("Error: " & ex.Message)
            Return Nothing
        End Try
    End Function

End Module
