﻿Imports System.ComponentModel
Imports System.Configuration.Install
Imports System.Security
Imports System.Security.Policy

Public Class Installer
    ' Private Variablen - hier werden die Details festgelegt
    Private ReadOnly PolLevel_Install As String = "User"
    Private ReadOnly Permset_Install As String = "FullTrust"
    Private ReadOnly CodeGroupDesc_Install As String = "VSTO_CASTest_2003-Group"
    Private ReadOnly ProduktName As String = "VSTO_CASTest_2003"
    Private ReadOnly AppName As String = My.Application.Info.Title
    Private mCodeGroupName_Install As String = "VSTO_CASTest_2003"

    Public Sub New()
        MyBase.New()

        'Dieser Aufruf ist für den Komponenten-Designer erforderlich.
        InitializeComponent()

        'Initialisierungscode nach dem Aufruf von InitializeComponent hinzufügen

    End Sub

    Public Overrides Sub Commit(ByVal savedState As System.Collections.IDictionary)
        MyBase.Commit(savedState)
    End Sub

    ' Wird bei der Installation aufgerufen
    Public Overrides Sub Install(ByVal stateSaver As IDictionary)
        Try
            SetupCAS()
            'MessageBox.Show("Install hat fertig...")
            MyBase.Install(stateSaver)
        Catch ex As SystemException
            LogError("Install", ex)
            Me.Rollback(stateSaver)
        End Try
    End Sub

    ' Wird bei der Deinstallation aufgerufen
    Public Overrides Sub Uninstall(ByVal savedState As IDictionary)
        Try
            Me.RemoveCAS()
        Catch Ex As SystemException
            LogError("UnInstall", Ex)
            MyBase.Uninstall(savedState)
        End Try
    End Sub

    ' Liefert das Installationsverzeichnis
    Private ReadOnly Property InstallDirectory() As String
        Get
            ' Installationsverzeichnis des Installers abfragen – AssInstallPfad
            Dim AssPfad As String = Me.Context.Parameters("AssInstallPfad")
            Dim InstallDir As String = AssPfad.Substring(0, AssPfad.LastIndexOf("\"))
            MessageBox.Show("InstallDirectory=" & InstallDir)
            'System.Diagnostics.Debugger.Break()

            EventLog.WriteEntry(AppName, "Install-Directory: " & AssPfad, EventLogEntryType.Information)
            If Not InstallDir.EndsWith("\") Then
                InstallDir &= "\"
            End If
            InstallDir &= "*"
            Return InstallDir
        End Get
    End Property

    ' Prüft, ob die anzulegende Codegruppe bereits existiert und gibt diese zurück, sollte dies der Fall sein
    Private Function GetCodeGroup(ByVal PolLevel As PolicyLevel) As CodeGroup
        Try
            EventLog.WriteEntry(AppName, "GetCodeGroup wird ausgeführt.", EventLogEntryType.Information)
            Dim tmpCodeGroup As CodeGroup
            For Each tmpCodeGroup In PolLevel.RootCodeGroup.Children
                If (tmpCodeGroup.Name.CompareTo(mCodeGroupName_Install) = 0) Then
                    Return tmpCodeGroup
                End If
            Next
            Return Nothing
        Catch ex As SystemException
            ' Es trat ein Fehler auf
            LogError("GetCodeGroup", ex)
            Return Nothing
        End Try
    End Function


    ' Richtet die CAS-Policy ein
    Sub SetupCAS()
        Try
            EventLog.WriteEntry(AppName, "SetupCAS wird ausgeführt", EventLogEntryType.Information)
            Dim PolLev As PolicyLevel = GetPolLevel()
            If GetCodeGroup(PolLev) Is Nothing Then
                ' Neuen FullTrust-Permissionset anlegen
                Dim PermSet As PermissionSet = New NamedPermissionSet(Me.Permset_Install)
                Dim MembershipCond As IMembershipCondition = New UrlMembershipCondition(InstallDirectory)
                ' Jetzt die Gruppe anlegen
                Dim PolStat As New PolicyStatement(PermSet)
                Dim CodeGruppe As CodeGroup = New UnionCodeGroup(MembershipCond, PolStat)
                CodeGruppe.Description = Me.CodeGroupDesc_Install
                CodeGruppe.Name = mCodeGroupName_Install
                ' Neue Codegruppe zum PolicyLevel hinzufügen
                PolLev.RootCodeGroup.AddChild(CodeGruppe)
                ' Alles speichern
                SecurityManager.SavePolicy()
            End If
        Catch ex As SystemException
            ' Es trat ein Fehler auf
            LogError("SetupCAS", ex)
        End Try
    End Sub

    ' Liefert das PolicyLevel-Objekt, z.B. User
    Private Function GetPolLevel() As PolicyLevel
        Try
            ' Den Level erst lokalisieren
            EventLog.WriteEntry(AppName, "GetPolLevel wird ausgeführt", EventLogEntryType.Information)
            Dim tmpPolLevel As PolicyLevel = Nothing
            Dim AllePolLevel As IEnumerator = SecurityManager.PolicyHierarchy()
            While AllePolLevel.MoveNext()
                Dim PolLevel As PolicyLevel = CType(AllePolLevel.Current, PolicyLevel)
                If (PolLevel.Label.CompareTo(Me.PolLevel_Install) = 0) Then
                    tmpPolLevel = PolLevel
                    Exit While
                End If
            End While
            If tmpPolLevel Is Nothing Then
                Throw New ApplicationException("User-Gruppe nicht gefunden - Probleme mit CAS-Konfiguration auf diesem Computer")
            End If
            Return tmpPolLevel
        Catch ex As SystemException
            ' Es trat ein Fehler auf
            LogError("GetPolLevel", ex)
            Return Nothing
        End Try
    End Function

    Sub RemoveCAS()
        Try
            EventLog.WriteEntry(AppName, "RemoveCAS wird ausgeführt", EventLogEntryType.Information)
            Dim PolLevel As PolicyLevel = GetPolLevel()
            Dim CodeGruppe As CodeGroup = GetCodeGroup(PolLevel)
            If CodeGruppe IsNot Nothing Then
                PolLevel.RootCodeGroup.RemoveChild(CodeGruppe)
                ' Nicht vergessen alles zu speichern
                SecurityManager.SavePolicy()
            End If
        Catch Ex As SystemException
            ' Es trat ein Fehler auf
            LogError("RemoveCAS", ex)
        End Try
    End Sub

    ' Fehler im Eventlog protokollieren
    Sub LogError(ByVal QUelle As String, ByVal Ex As SystemException)
        EventLog.WriteEntry(My.Application.Info.Title, "Error in " & Quelle & _
      "Error-Message: " & Ex.Message, EventLogEntryType.Error)
    End Sub

End Class
