﻿Public Class nodeProperties
    Private mActiveDocument As Word.Document
    Private ndActual As Word.XMLNode

    Public WriteOnly Property activeDocument() As Word.Document
        Set(ByVal value As Word.Document)
            mActiveDocument = value
        End Set
    End Property

    Private Sub nodeProperties_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        Try
            ndActual = mActiveDocument.Application.Selection.Range.XMLParentNode
            If Not ndActual Is Nothing Then
                ndActual.Range.Select()
                actualizeInfos(ndActual)
            Else
                ndActual = mActiveDocument.XMLNodes.Item(1)
                If Not ndActual Is Nothing Then
                    ndActual.Range.Select()
                    actualizeInfos(ndActual)
                End If
            End If
        Catch ex As Exception
            MsgBox(ex.Message)
        End Try
    End Sub

    Private Sub actualizeInfos(ByVal nd As Word.XMLNode)
        lblBaseName.Text = ndActual.BaseName
        lblNamespaceURI.Text = ndActual.NamespaceURI
        lblText.Text = ndActual.Text
        Select Case ndActual.ValidationStatus
            Case Word.WdXMLValidationStatus.wdXMLValidationStatusCustom
                lblValidationStatus.Text = "wdXMLValidationStatusCustom"
            Case Word.WdXMLValidationStatus.wdXMLValidationStatusOK
                lblValidationStatus.Text = "wdXMLValidationStatusOK"
        End Select
        Select Case ndActual.Level
            Case Word.WdXMLNodeLevel.wdXMLNodeLevelCell
                lblLevel.Text = "wdXMLNodeLevelCell"
            Case Word.WdXMLNodeLevel.wdXMLNodeLevelInline
                lblLevel.Text = "wdXMLNodeLevelInline"
            Case Word.WdXMLNodeLevel.wdXMLNodeLevelParagraph
                lblLevel.Text = "wdXMLNodeLevelParagraph"
            Case Word.WdXMLNodeLevel.wdXMLNodeLevelRow
                lblLevel.Text = "wdXMLNodeLevelRow"
        End Select
        Dim strXPath As String = ""
        strXPath = "/" & ndActual.BaseName
        Dim ndParent As Word.XMLNode = ndActual.ParentNode
        While Not ndParent Is Nothing
            strXPath = "/" & ndParent.BaseName & strXPath
            ndParent = ndParent.ParentNode
        End While
        lblXPath.Text = strXPath
        txtPlaceholderText.Text = ndActual.PlaceholderText
        txtValidationErrorText.Text = ndActual.ValidationErrorText
        DataGridView1.Rows.Clear()
        Dim attr As Word.XMLNode
        Dim intRow As Integer = 0
        For Each attr In ndActual.Attributes
            intRow = DataGridView1.Rows.Add()
            DataGridView1.Rows(intRow).Cells(colAttribute.Name).Value = attr.BaseName
            DataGridView1.Rows(intRow).Cells(colValue.Name).Value = attr.NodeValue
        Next
    End Sub

    Private Sub btnParentNode_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnParentNode.Click
        If Not ndActual Is Nothing Then
            Try
                ndActual = ndActual.ParentNode
                If ndActual Is Nothing Then
                    ndActual = mActiveDocument.XMLNodes(1)
                End If
                If Not ndActual Is Nothing Then
                    ndActual.Range.Select()
                    actualizeInfos(ndActual)
                End If
            Catch ex As Exception

            End Try
        End If
    End Sub

    Private Sub btnCildNode_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnCildNode.Click
        If Not ndActual Is Nothing Then
            If ndActual.ChildNodes.Count > 0 Then
                ndActual = ndActual.ChildNodes(1)
            End If
            ndActual.Range.Select()
            actualizeInfos(ndActual)
        End If
    End Sub

    Private Sub btnNextNode_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnNextNode.Click
        If Not ndActual Is Nothing Then
            If Not ndActual.NextSibling Is Nothing Then
                ndActual = ndActual.NextSibling
                ndActual.Range.Select()
                actualizeInfos(ndActual)
            End If
        End If
    End Sub

    Private Sub btnPreviousNode_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnPreviousNode.Click
        If Not ndActual Is Nothing Then
            If Not ndActual.PreviousSibling Is Nothing Then
                ndActual = ndActual.PreviousSibling
                ndActual.Range.Select()
                actualizeInfos(ndActual)
            End If
        End If
    End Sub

    Private Sub btnApplyPlaceholderText_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnApplyPlaceholderText.Click
        If Not ndActual Is Nothing Then
            ndActual.PlaceholderText = txtPlaceholderText.Text
        End If
    End Sub

    Private Sub btnApplyValdationErrortext_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnApplyValdationErrortext.Click
        If Not ndActual Is Nothing Then
            ndActual.SetValidationError(Word.WdXMLValidationStatus.wdXMLValidationStatusCustom, txtValidationErrorText.Text.ToString, True)
        End If
    End Sub

    Private Sub btnCopyXPath_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnCopyXPath.Click
        Windows.Forms.Clipboard.SetDataObject(lblXPath.Text)
    End Sub

    Private Sub btnApplyAttributes_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnApplyAttributes.Click
        For Each r As Windows.Forms.DataGridViewRow In DataGridView1.Rows
            ndActual.Attributes(r.Index + 1).NodeValue = r.Cells(colValue.Name).Value.ToString
        Next
    End Sub
End Class